using System;
using System.IO;
using System.IO.Ports;
using ChuPiao.AutoPrint.Machine.Models;
using ChuPiao.AutoPrint.Machine.Printer;
using ChuPiao.Common.Core.Utils;
using ChuPiao.Common.Models;
using log4net;
using ChuPiao.Common.Utils;

namespace ChuPiao.AutoPrint.Machine
{
    public abstract class BaseVirtualPrinter:IVirtualPrinter
    {
        #region Implementation of IVirtualPrinter

        private VirtualPrinterStatus _status;

        private ILog _log;

        private string _logPath;

        private SerialPort _printerPort;

        private string _port;

        private int _portBps;

        private M_MachineEntity _machineInfo;

        private int _lastPrintCounter;

        private string _lastPrintFilePath;

        private ILotMachine _lotMachine;

        /// <summary>
        /// 
        /// </summary>
        public ILotMachine LotMachine
        {
            get { return _lotMachine; }
            set { _lotMachine = value; }
        }

        /// <summary>
        /// ӡ״̬
        /// </summary>
        public VirtualPrinterStatus Status
        {
            get { return _status; }
            set { _status=value; }
        }

        /// <summary>
        /// ־¼
        /// </summary>
        public ILog Log
        {
            get { return _log; }
            set { _log = value; }
        }

        /// <summary>
        /// ־·־вƱĵƱļȶ·棬ʽ磺C:\example\
        /// </summary>
        public string LogPath
        {
            get { return _logPath; }
            set { _logPath = value; }
        }

        /// <summary>
        /// ˿
        /// </summary>
        public SerialPort PrinterPort
        {
            get { return _printerPort; }
            set { _printerPort = value; }
        }

        /// <summary>
        /// ˿ں
        /// </summary>
        public string Port
        {
            get { return _port; }
            set { _port = value; }
        }

        /// <summary>
        /// ˿ڲ
        /// </summary>
        public int PortBps
        {
            get { return _portBps; }
            set { _portBps = value; }
        }

        /// <summary>
        /// ƱϢ
        /// </summary>
        public M_MachineEntity MachineInfo
        {
            get { return _machineInfo; }
            set { _machineInfo = value; }
        }
        private IPrinterClient _printerClient; 
        /// <summary>
        /// Ʊӡ
        /// </summary>
        public IPrinterClient PrinterClient
        {
            get { return _printerClient; }
            set { _printerClient = value; }
        }

        /// <summary>
        /// ʼ
        /// </summary>
        public abstract void Initialize();

        /// <summary>
        /// 
        /// </summary>
        public abstract void Start();

        /// <summary>
        /// ֹͣ
        /// </summary>
        public abstract void Stop();

        /// <summary>
        /// Ʊ
        /// </summary>
        /// <returns></returns>
        public abstract CaptureDataInfo Output();

        /// <summary>
        /// Ʊ
        /// </summary>
        /// <returns></returns>
        public abstract CaptureDataInfo ReOutput();

        /// <summary>
        /// ǿֹͣǰƱ
        /// </summary>
        public abstract void ForceStop();

        /// <summary>
        /// սջ
        /// </summary>
        public abstract void ClearBuffer();

        /// <summary>
        /// ӡ
        /// </summary>
        public int LastPrintCounter
        {
            get { return _lastPrintCounter; }
            set { _lastPrintCounter = value; }
        }

        /// <summary>
        /// Ĵӡļ·
        /// </summary>
        public string LastPrintFilePath
        {
            get { return _lastPrintFilePath; }
            set { _lastPrintFilePath = value; }
        }

        #endregion

        protected string AlldataFilePath
        {
            get
            {
                string ticketFilePath = Path.Combine(LogPath + DateTime.Now.ToString("yyyy-MM-dd"), "alldata");
                if (!Directory.Exists(ticketFilePath))
                {
                    Directory.CreateDirectory(ticketFilePath);
                }
                return Path.Combine(ticketFilePath, MachineInfo.M_Number + "_alldata.prn");
            }
        }

        public void SaveLastFile(CaptureDataInfo captureDataInfo)
        {
            SaveLastFile(captureDataInfo.CaptureDataBytes);
            //try
            //{
            //    LastPrintCounter++;
            //    if (LastPrintCounter > 5000)
            //    {
            //        LastPrintCounter = 1;
            //    }
            //    string ticketFilePath = Path.Combine(LogPath + DateTime.Now.ToString("yyyy-MM-dd"), "last");
            //    if (!Directory.Exists(ticketFilePath))
            //    {
            //        Directory.CreateDirectory(ticketFilePath);
            //    }
            //    ticketFilePath = Path.Combine(ticketFilePath, LastPrintCounter + ".prn");
            //    LastPrintFilePath = ticketFilePath;
            //    File.WriteAllText(ticketFilePath, TicketTextUtil.CompressAndEncrypt(captureDataInfo.CaptureDataBytes));
            //    Log.Info(string.Concat("Lastļ", ticketFilePath));
            //}
            //catch (Exception exception)
            //{
            //    Log.Warn(string.Concat("Lastļ쳣Ϣ", exception.ToString()));
            //}
        }
        public void SaveLastFile(byte[] captureDataBytes)
        {
            try
            {
                //LastPrintCounter++;
                //if (LastPrintCounter > 1000)
                //{
                //    LastPrintCounter = 1;
                //}
                string ticketFilePath = Path.Combine(LogPath + DateTime.Now.ToString("yyyy-MM-dd"), "last");
                if (!Directory.Exists(ticketFilePath))
                {
                    Directory.CreateDirectory(ticketFilePath);
                }
                ticketFilePath = Path.Combine(ticketFilePath, DateTime.Now.ToString("yyyyMMddHHmmss") + ".prn");
                LastPrintFilePath = ticketFilePath;
                File.WriteAllText(ticketFilePath, TicketTextUtil.CompressAndEncrypt(captureDataBytes));
                //Log.Debug(string.Concat("Lastļ", ticketFilePath));
                Log.Debug("Lastļ");
            }
            catch (Exception exception)
            {
                Log.Warn(string.Concat("Lastļ쳣Ϣ", exception.ToString()));
            }
        }

        public void DeleteLastFile()
        {
            try
            {
                string ticketFilePath = Path.Combine(LogPath + DateTime.Now.ToString("yyyy-MM-dd"), "last");
                if (Directory.Exists(ticketFilePath))
                {
                    DirectoryInfo directoryInfo = new DirectoryInfo(ticketFilePath);
                    FileInfo[] fileInfoList = directoryInfo.GetFiles("*.prn");

                    if (fileInfoList.Length == 0)
                        return;

                    LastTicketFileComparer fileComparer = new LastTicketFileComparer();
                    Array.Sort(fileInfoList, fileComparer);

                    FileInfo file = fileInfoList[0];
                    file.Delete();
                    //Log.Debug(string.Concat("ɾLastļ", file.FullName));
                    Log.Debug("ɾLastļ");
                }
            }
            catch (Exception exception)
            {
                Log.Debug(string.Concat("ɾLastļ쳣Ϣ", exception.ToString()));
            }
        }

    }
}
