using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using log4net;

namespace ChuPiao.Common.Core.Socket
{
    /// <summary>
    /// SocketͨѶͻ
    /// </summary>
    public class SocketClient : ISocketClient
    {
        private IPEndPoint _remoteIpEndPoint;
        private System.Net.Sockets.Socket _socket;
        private int _receiveTimeout;
        private int _sendTimeout;
        private int _pingIdleTime;
        private string _pingMsg;
        private Thread _pingThread;
        private ILog _logger;
        private bool _pingIsRun;
        private int _retryNum = 3;
        private NetworkStream _networkStream;
        private Encoding _charset;
        private bool _isStart;

        //ʱ
        private volatile int _lastTranslation;

        public SocketClient(string host, int port,int receiveTimeout,int sendTimeout,int pingIdleTime,string pingMsg,ILog logger)
        {
            _remoteIpEndPoint = new IPEndPoint(IPAddress.Parse(host), port);
            _receiveTimeout = receiveTimeout;
            _sendTimeout = sendTimeout;
            _pingIdleTime = pingIdleTime;
            _pingMsg = pingMsg;
            _logger = logger;
            _charset = Encoding.UTF8;

            _isStart = false;

            _pingIsRun = true;
            _pingThread = new Thread(new ThreadStart(DoPing));
            _pingThread.IsBackground = true;
        }
        /// <summary>
        /// ping
        /// </summary>
        private void DoPing()
        {
            while (_pingIsRun)
            {
                //˯ʱʱ
                int delay=0;
                if(!Connected)
                {
                    lock (this)
                    {
                        ReConnect();
                    }
                    delay = 0;
                }
                else
                {
                    delay = _pingIdleTime - (Environment.TickCount - _lastTranslation);
                    if(delay<=0)
                    {
                        //_logger.Debug("ӿУҪping");
                        SendPing();
                    }
                }
                try
                {
                    if(delay>0)
                    {
                        Thread.Sleep(delay);
                    }
                    else
                    {
                        Thread.Sleep(_pingIdleTime);
                    }
                }
                catch (ThreadInterruptedException e)
                {
                }
            }
        }

        private void SendPing()
        {
            lock (this)
            {
                try
                {
                    if(!Connected)
                    {
                        ReConnect();
                        return;
                    }
                    int delay = _pingIdleTime - (Environment.TickCount - _lastTranslation);
                    if(delay>0)
                    {
                        _logger.Debug("PingȴʱͣҪPing");
                        return;
                    }
                    Console.WriteLine("");
                    Send(_pingMsg);
                    Read();
                    _logger.Debug("pingɹ");
                }
                catch (Exception e)
                {
                    bool isIOException = e is IOException;
                    if(!isIOException)
                    {
                        _logger.Info(string.Concat("IO쳣ԭSocketClient½ӣ",e));
                    }
                    Console.WriteLine(string.Concat("쳣Ϣ", e));
                    ReConnect();
                }
            }
        }
        /// <summary>
        /// Ϣ
        /// </summary>
        /// <param name="message"></param>
        private void Send(string message)
        {
            //Console.WriteLine(string.Concat("Ϣ",message));
            byte[] contentData = _charset.GetBytes(message);
            //ΪBitConverter.GetBytesǰСֽתbyte飬ֽǴֽԽת
            byte[] headerData = BitConverter.GetBytes(IPAddress.HostToNetworkOrder(contentData.Length));
            _networkStream.Write(headerData,0,headerData.Length);
            _networkStream.Write(contentData,0,contentData.Length);
            _networkStream.Flush();
        }
        /// <summary>
        /// ȡظϢ
        /// </summary>
        /// <returns></returns>
        private string Read()
        {
            string content = string.Empty;
            byte[] buffer = new byte[4];
            _networkStream.Read(buffer, 0, buffer.Length);
            //ΪBitConverter.GetBytesǰСֽתbyte飬ֽǴֽԽת
            int contentLength = IPAddress.NetworkToHostOrder(BitConverter.ToInt32(buffer, 0));
            if (contentLength > 0)
            {
                byte[] contentData = new byte[contentLength];

                int offset = 0;
                int num = 0;
                do
                {
                    num = _networkStream.Read(contentData, offset, contentLength - offset);
                    offset += num;
                } while (num>0&&offset < contentLength);
                if(offset!=contentLength)
                {
                    throw new Exception("װʵʽݳȲ");
                }
                _lastTranslation = Environment.TickCount;
                content = _charset.GetString(contentData);
            }
            return content;
        }

        /// <summary>
        /// ͻ
        /// </summary>
        private void Start()
        {
            try
            {
                Connect();
            }
            catch (IOException e)
            {
                _logger.Error(e);
            }
            _pingIsRun = true;
            if(_pingThread==null)
            {
                _pingThread = new Thread(new ThreadStart(DoPing));
                _pingThread.IsBackground = true;
            }
            _pingThread.Start();
            _isStart = true;
        }

        /// <summary>
        /// ӵԶ̷
        /// </summary>
        private void Connect()
        {
            for (int i = 0; i < _retryNum; i++)
            {
                try
                {
                    _socket = new System.Net.Sockets.Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                    _socket.SendTimeout = _sendTimeout;
                    _socket.ReceiveTimeout = _receiveTimeout;
                    _socket.NoDelay = true;
                    _socket.Connect(_remoteIpEndPoint);
                    _logger.Info(string.Concat("ӵ",_remoteIpEndPoint.ToString(),"ɹ"));
                    _lastTranslation = Environment.TickCount;//ʱ
                    if (_socket.Connected)
                    {
                        break;
                    }
                }
                catch (Exception exception)
                {
                    _logger.Info(string.Concat("ӵ", _remoteIpEndPoint.ToString(), "쳣Ϣ",exception));
                    Console.WriteLine(string.Concat("쳣", exception));

                    if(i==_retryNum-1)
                    {
                        throw new IOException(string.Concat("ӵ",_remoteIpEndPoint.ToString()," ",_retryNum,"ʧ"));
                    }
                    _logger.Info(string.Concat("Ե",i+1, "ӵ", _remoteIpEndPoint.ToString()));
                }
            }
            _networkStream = new NetworkStream(_socket, true);
        }
        /// <summary>
        /// 
        /// </summary>
        private void ReConnect()
        {
            _logger.Info("Ӷϵڽ");
            Console.WriteLine("Ӷϵڽ");
            try
            {
                _socket.Close();
            }
            catch (IOException e)
            {
                _logger.Error(e);
            }
            try
            {
                Connect();
            }
            catch (IOException e)
            {
                _logger.Error(e);
            }
        }
        /// <summary>
        /// ͻǷ״̬
        /// </summary>
        private bool Connected
        {
            get { return _socket != null && _socket.Connected; }
        }

        #region Implementation of ISocketClient

        /// <summary>
        /// ύ
        /// </summary>
        /// <param name="requestMsg"></param>
        /// <returns></returns>
        public string CommitRequest(string requestMsg)
        {
            if(!_isStart)
            {
                lock (this)
                {
                    if(!_isStart)
                    {
                        Start();
                    }
                }
            }
            if(!Connected)
            {
                throw new IOException(string.Concat("ȴSocketClient ,ʱ䣺" , _pingIdleTime , ""));
            }
            lock (this)
            {
                if (!Connected)
                {
                    throw new IOException(string.Concat("ȴSocketClient ,ʱ䣺", _pingIdleTime, ""));
                }
                try
                {
                    Send(requestMsg);
                    return Read();
                }
                catch (Exception e)
                {
                    bool isIOException = e is IOException;
                    if (!isIOException)
                    {
                        _logger.Info(string.Concat("IO쳣ԭSocketClientرӣ", e));
                    }
                    try
                    {
                        _socket.Close();
                    }
                    catch (Exception ex)
                    {
                        _logger.Error(ex);
                    }
                    _pingThread.Interrupt();
                    _pingThread = null;
                    _isStart = false;
                    throw;
                }
            }
        }

        /// <summary>
        /// رտͻ
        /// </summary>
        public void Close()
        {
            _pingIsRun = false;
            if (_pingThread!=null)
            {
                _pingThread.Interrupt();
            }
            _pingThread = null;
            if(_socket!=null)
            {_socket.Close();}
            _isStart = false;
        }

        #endregion
    }
}

